@@ -1463,34 +1463,31 @@ static double s_pow(double x, double y) {
   } else
     return pow(x, y);
 }
-#elif defined(MACOSX)
-/* intel macosx delivers precise results for integer inputs, e.g.,
- * 10.0^21.0, only with long double version of pow */
+#elif (machine_type == machine_type_i3osx || machine_type == machine_type_ti3osx)
+/* intel macosx delivers accurate results for integer inputs, e.g.,
+ * 10.0^21.0, only with long double version of pow; it's not clear
+ * whether that has been true for x86_64 as opposed to i386, but as
+ * of macOS 10.15 (Catalina), pow seems ok for x86_64 */
 static double s_pow(double x, double y) { return powl(x, y); }
-#else /* i3fb/ti3fb */
+#else /* i3fb/ti3fb/i3osx/ti3osx */
 static double s_pow(double x, double y) { return pow(x, y); }
-#endif /* i3fb/ti3fb */
+#endif /* i3fb/ti3fb/i3osx/ti3osx */
 
 #ifdef __MINGW32__
-/* cos() and sin() do not handle large values nicely,
-   so use fmod() to get reasonably close */
-# define INTO_SINCOS_RANGE(x) (((x > 1e9) || (x < -1e9)) \
-			       ? fmod(x, 2*atan2(0.0, -1.0)) \
-			       : x)
 /* asinh() and atanh() sometimes get zero sign wrong */
 # define CHECK_ASINTAN_ZERO(x, e) ((x == 0.0) \
                                    ? (signbit(x) ? -0.0 : 0.0)  \
                                    : e)
+/* see also "mingw.zuo" */
 #else
-# define INTO_SINCOS_RANGE(x) x
 # define CHECK_ASINTAN_ZERO(x, e) e
 #endif
 
 static double s_sqrt(double x) { return sqrt(x); }
 
-static double s_sin(double x) { return sin(INTO_SINCOS_RANGE(x)); }
+static double s_sin(double x) { return sin(x); }
 
-static double s_cos(double x) { return cos(INTO_SINCOS_RANGE(x)); }
+static double s_cos(double x) { return cos(x); }
 
 static double s_tan(double x) { return tan(x); }
 
