@@ -1343,6 +1343,7 @@ enum {
   MZCONFIG_ERROR_PRINT_VALUE_HANDLER,
   MZCONFIG_ERROR_PRINT_SYNTAX_HANDLER,
   MZCONFIG_ERROR_NAME_SYNTAX_HANDLER,
+  MZCONFIG_ERROR_PRINT_MODULE_PATH_HANDLER,
   MZCONFIG_ERROR_MESSAGE_ADJUSTER,
 
   MZCONFIG_EXIT_HANDLER,
@@ -1949,6 +1950,7 @@ MZ_EXTERN void scheme_set_collects_path(Scheme_Object *p);
 MZ_EXTERN void scheme_set_config_path(Scheme_Object *p);
 MZ_EXTERN void scheme_set_host_collects_path(Scheme_Object *p);
 MZ_EXTERN void scheme_set_host_config_path(Scheme_Object *p);
+MZ_EXTERN void scheme_set_host_addon_dir(Scheme_Object *p);
 MZ_EXTERN void scheme_set_original_dir(Scheme_Object *d);
 MZ_EXTERN void scheme_set_addon_dir(Scheme_Object *p);
 MZ_EXTERN void scheme_set_command_line_arguments(Scheme_Object *vec);
