@@ -42,6 +42,7 @@ static void init_extfl(Scheme_Startup_Env *env);
 static void init_futures(Scheme_Startup_Env *env);
 static void init_foreign(Scheme_Startup_Env *env);
 static void init_terminal(Scheme_Startup_Env *env);
+static void init_internal(Scheme_Startup_Env *env);
 
 static void skip_certain_things(Scheme_Object *o, Scheme_Close_Custodian_Client *f, void *data);
 
@@ -318,6 +319,7 @@ static void init_startup_env(void)
   init_extfl(env);
   init_futures(env);
   init_terminal(env);
+  init_internal(env);
 
   builtin_unsafe_start = builtin_ref_counter;
   scheme_init_unsafe_linklet(env);
@@ -412,6 +414,15 @@ static void init_terminal(Scheme_Startup_Env *env)
   scheme_restore_prim_instance(env);
 }
 
+static void init_internal(Scheme_Startup_Env *env)
+{
+  scheme_switch_prim_instance(env, "#%internal");
+
+  scheme_init_internal_string(env);
+
+  scheme_restore_prim_instance(env);
+}
+
 /*========================================================================*/
 /*                     place-specific initialization                       */
 /*========================================================================*/
