@@ -107,6 +107,7 @@ extern MZGC_DLLIMPORT void GC_init();
 void scheme_set_stack_base(void *base, int no_auto_statics) XFORM_SKIP_PROC
 {
 #ifdef MZ_PRECISE_GC
+  GC_report_signal_handle_modify = rktio_will_modify_os_signal_handler;
   GC_init_type_tags(_scheme_last_type_, 
                     scheme_pair_type, scheme_mutable_pair_type, scheme_weak_box_type, 
                     scheme_ephemeron_type, scheme_rt_weak_array,
@@ -461,9 +462,18 @@ void scheme_set_signal_handler(int sig_id, Scheme_Signal_Handler_Proc proc) XFOR
 {
 #ifndef WINDOWS_PROCESSES
   struct sigaction sa;
+
+  if (sig_id == SIGHUP) {
+    /* cooperate with `nohup` by keeping SIHGUP as-is when ignored */
+    sigaction(SIGHUP, NULL, &sa);
+    if (sa.sa_handler == SIG_IGN)
+      return;
+  }
+
   sigemptyset(&sa.sa_mask);
   sa.sa_flags = 0;
   sa.sa_handler = (proc ? proc : SIG_IGN);
+  rktio_will_modify_os_signal_handler(sig_id);
   sigaction(sig_id, &sa, NULL);
 #endif
 }
@@ -947,6 +957,8 @@ THREAD_LOCAL_DECL(intptr_t scheme_code_page_total);
 THREAD_LOCAL_DECL(intptr_t scheme_code_total);
 THREAD_LOCAL_DECL(intptr_t scheme_code_count);
 
+THREAD_LOCAL_DECL(int jit_code_write_enabled);
+
 #if defined(MZ_CODE_ALLOC_USE_MPROTECT) && !defined(MAP_ANON)
 static int fd, fd_created;
 #endif
@@ -957,6 +969,22 @@ static int fd, fd_created;
 # define MAYBE_PROT_EXEC PROT_EXEC
 #endif
 
+#ifdef MZ_USE_MAP_JIT
+# define MAYBE_MAP_JIT MAP_JIT
+void scheme_thread_code_start_write(void) {
+  if (jit_code_write_enabled == 0)
+    pthread_jit_write_protect_np(0);
+  jit_code_write_enabled++;
+}
+void scheme_thread_code_end_write(void) {
+  --jit_code_write_enabled;
+  if (jit_code_write_enabled == 0)
+    pthread_jit_write_protect_np(1);
+}
+#else
+# define MAYBE_MAP_JIT 0
+#endif
+
 #define LOG_CODE_MALLOC(lvl, s) /* if (lvl > 1) s */
 #define CODE_PAGE_OF(p) ((void *)(((uintptr_t)p) & ~(page_size - 1)))
 
@@ -1031,13 +1059,13 @@ static void *malloc_page(intptr_t size)
   }
 #else
 # ifdef MAP_ANON
-  r = mmap(NULL, size, PROT_READ | PROT_WRITE | MAYBE_PROT_EXEC, MAP_PRIVATE | MAP_ANON, -1, 0);
+  r = mmap(NULL, size, PROT_READ | PROT_WRITE | MAYBE_PROT_EXEC, MAP_PRIVATE | MAYBE_MAP_JIT | MAP_ANON, -1, 0);
 # else
   if (!fd_created) {
     fd_created = 1;
     fd = open("/dev/zero", O_RDWR);
   }
-  r = mmap(NULL, size, PROT_READ | PROT_WRITE | MAYBE_PROT_EXEC, MAP_PRIVATE, fd, 0);
+  r = mmap(NULL, size, PROT_READ | PROT_WRITE | MAYBE_PROT_EXEC, MAP_PRIVATE | MAYBE_MAP_JIT, fd, 0);
 # endif
   if (r  == (void *)-1)
     r = NULL;
@@ -1144,6 +1172,8 @@ void *scheme_malloc_code(intptr_t size)
   intptr_t size2, bucket, sz, page_size;
   void *p, *pg, *prev;
 
+  scheme_thread_code_start_write();
+
   if (size < CODE_HEADER_SIZE) {
     /* ensure CODE_HEADER_SIZE alignment 
        and room for free-list pointers */
@@ -1213,6 +1243,8 @@ void *scheme_malloc_code(intptr_t size)
     LOG_CODE_MALLOC(0, printf("allocated %ld (->%ld / %ld)\n", size, size2, bucket));
   }
 
+  scheme_thread_code_end_write();
+
   return p;
 #else
   return malloc(size); /* good luck! */
@@ -1267,6 +1299,8 @@ void scheme_free_code(void *p)
   int per_page, n;
   void *prev;
 
+  scheme_thread_code_start_write();
+
   page_size = get_page_size();
 
   size = *(intptr_t *)CODE_PAGE_OF(p);
@@ -1345,6 +1379,8 @@ void scheme_free_code(void *p)
     }
   }
 
+  scheme_thread_code_end_write();
+
 #else
   free(p);
 #endif
@@ -1356,6 +1392,8 @@ void scheme_free_all_code(void)
   void *p, *next;
   intptr_t page_size;
 
+  scheme_thread_code_start_write();
+
   page_size = get_page_size();
 
   for (p = code_allocation_page_list; p; p = next) {
@@ -1368,6 +1406,8 @@ void scheme_free_all_code(void)
   code_allocation_page_list = NULL;
 
   free_page(free_list, page_size);
+
+  scheme_thread_code_end_write();
 #endif
 }
 
@@ -1625,6 +1665,7 @@ static void install_w_xor_x_handler()
     sigaddset(&act.sa_mask, SIGINT);
     sigaddset(&act.sa_mask, SIGCHLD);
     act.sa_flags = SA_SIGINFO;
+    rktio_will_modify_os_signal_handler(SIG_W_XOR_X);
     sigaction(SIG_W_XOR_X, &act, &oact);
     previous_fault_handler = oact.sa_sigaction;
   }
